@@ -2,7 +2,8 @@
 #
 # Copyright 2007 Sun Microsystems Inc.
 #
-# This library is free software; you can redistribute it and/or# modify it under the terms of the GNU Library General Public
+# This library is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Library General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
@@ -24,7 +25,6 @@ __date__      = "$Date:$"
 __copyright__ = "Copyright (c) 2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
-import gettext
 import gtk
 import locale
 import os
@@ -38,7 +38,6 @@ import focus_tracking_presenter
 import input_event
 import keybindings
 import orca_gui_prefs
-import orca_prefs
 import orca_state
 import platform
 import settings
@@ -52,30 +51,30 @@ OS = None
 
 class orcaSetupGUI(orca_gui_prefs.orcaSetupGUI):
 
-    def _initAppGUIState(self, appScript):
+    def _initAppGUIState(self, thisAppScript):
         """Before we show the GUI to the user we want to remove the
         General tab and gray out the Speech systems and servers 
         controls on the speech tab.
 
         Arguments:
-        - appScript: the application script.
+        - thisAppScript: the application script.
         """
 
         # Save away the application script so that it can be used later
         # by writeUserPreferences().
         #
-        self.appScript = appScript
+        self.appScript = thisAppScript
 
-        self.notebook.remove_page(0)
-        self.speechSystemsLabel.set_sensitive(False)
-        self.speechSystems.set_sensitive(False)
-        self.speechServersLabel.set_sensitive(False)
-        self.speechServers.set_sensitive(False)
+        self.get_widget("notebook").remove_page(0)
+        self.get_widget("speechSystemsLabel").set_sensitive(False)
+        self.get_widget("speechSystems").set_sensitive(False)
+        self.get_widget("speechServersLabel").set_sensitive(False)
+        self.get_widget("speechServers").set_sensitive(False)
 
-        vbox = appScript.getAppPreferencesGUI()
+        vbox = self.appScript.getAppPreferencesGUI()
         if vbox:
-            label = gtk.Label(orca_state.locusOfFocus.app.name)
-            self.notebook.append_page(vbox, label)
+            label = gtk.Label(orca_state.locusOfFocus.getApplication().name)
+            self.get_widget("notebook").append_page(vbox, label)
 
     def _createPronunciationTreeView(self, pronunciations=None):
         """Create the pronunciation dictionary tree view for this specific 
@@ -98,10 +97,10 @@ class orcaSetupGUI(orca_gui_prefs.orcaSetupGUI):
         # Adjust the title of the app-specific Orca Preferences dialog to
         # include the name of the application.
         #
-        self.app = orca_state.locusOfFocus.app
+        self.app = orca_state.locusOfFocus.getApplication()
         self.applicationName = self.app.name 
         title = _("Orca Preferences for %s") % self.applicationName
-        self.orcaSetupWindow.set_title(title)
+        self.get_widget("orcaSetupWindow").set_title(title)
 
         orca_gui_prefs.orcaSetupGUI._showGUI(self)
 
@@ -199,7 +198,7 @@ class orcaSetupGUI(orca_gui_prefs.orcaSetupGUI):
         """
 
         self.applyButtonClicked(widget)
-        self.orcaSetupWindow.destroy()
+        self.get_widget("orcaSetupWindow").destroy()
 
     def windowDestroyed(self, widget):
         """Signal handler for the "destroyed" signal for the orcaSetupWindow
@@ -220,7 +219,8 @@ def showPreferencesUI():
 
     # There must be an application with focus for this to work.
     #
-    if not orca_state.locusOfFocus or not orca_state.locusOfFocus.app:
+    if not orca_state.locusOfFocus or \
+       not orca_state.locusOfFocus.getApplication():
         message = _("No application has focus.")
         braille.displayMessage(message)
         speech.speak(message)
@@ -230,7 +230,7 @@ def showPreferencesUI():
 
     # The name of the application that currently has focus.
     #
-    applicationName = orca_state.locusOfFocus.app.name
+    applicationName = orca_state.locusOfFocus.getApplication().name
 
     # Translators: Orca Preferences in this case, is a configuration GUI 
     # for allowing users to set application specific settings from within
