@@ -25,9 +25,9 @@ operations on the Accessible object, such as invoking default actions or
 moving the text caret.
 """
 
-__id__        = "$Id: braille.py 2602 2007-08-08 17:15:07Z shaeger $"
-__version__   = "$Revision: 2602 $"
-__date__      = "$Date: 2007-08-08 13:15:07 -0400 (Wed, 08 Aug 2007) $"
+__id__        = "$Id: braille.py 3167 2007-10-23 23:01:32Z joanied $"
+__version__   = "$Revision: 3167 $"
+__date__      = "$Date: 2007-10-23 19:01:32 -0400 (Tue, 23 Oct 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -36,7 +36,7 @@ log = logging.getLogger("braille")
 
 import signal
 
-import atspi
+import pyatspi
 
 # We'll use the official BrlAPI pythons (as of BrlTTY 3.8) if they
 # are available.  Otherwise, we'll fall back to our own bindings.
@@ -69,8 +69,6 @@ import orca_state
 import settings
 
 from orca_i18n import _                          # for gettext support
-from rolenames import getShortBrailleForRoleName # localized role names
-from rolenames import getLongBrailleForRoleName  # localized role names
 
 # If True, this module has been initialized.
 #
@@ -153,8 +151,8 @@ CMD_RESTARTBRL        = 0x43
 CMD_RESTARTSPEECH     = 0x44
 CMD_MAX               = 0x44
 
-BRL_FLG_REPEAT_INITIAL= 0x800000
-BRL_FLG_REPEAT_DELAY  = 0x400000
+BRL_FLG_REPEAT_INITIAL = 0x800000
+BRL_FLG_REPEAT_DELAY   = 0x400000
 
 # Common names for most used BrlTTY commands, to be shown in the GUI:
 # ATM, the ones used in default.py are:
@@ -327,15 +325,12 @@ class Component(Region):
         associated with this region.  Note that the zeroeth character may have
         been scrolled off the display."""
 
-        actions = self.accessible.action
-        if actions:
-            actions.doAction(0)
-        else:
-
-            # [[[WDW - HACK to do a mouse button 1 click if we have
-            # to.  For example, page tabs don't have any actions but
-            # we want to be able to select them with the cursor
-            # routing key.]]]
+        try:
+            action = self.accessible.queryAction()
+        except:
+            # Do a mouse button 1 click if we have to.  For example, page tabs
+            # don't have any actions but we want to be able to select them with
+            # the cursor routing key.
             #
             debug.println(debug.LEVEL_FINEST,
                           "braille.Component.processRoutingKey: no action")
@@ -343,6 +338,8 @@ class Component(Region):
                 eventsynthesizer.clickObject(self.accessible, 1)
             except:
                 debug.printException(debug.LEVEL_SEVERE)
+        else:
+            action.doAction(0)
 
 class Text(Region):
     """A subclass of Region backed by a Text object.  This Region will
@@ -375,9 +372,9 @@ class Text(Region):
             self.caretOffset = 0
             self.lineOffset = 0
             [string, startOffset, endOffset] = \
-                self.accessible.text.getTextAtOffset(
+                self.accessible.queryText().getTextAtOffset(
                     0,
-                    atspi.Accessibility.TEXT_BOUNDARY_LINE_START)
+                    pyatspi.TEXT_BOUNDARY_LINE_START)
 
         cursorOffset = min(self.caretOffset - self.lineOffset, len(string))
 
@@ -427,15 +424,16 @@ class Text(Region):
                 return
 
         newCaretOffset = min(self.lineOffset + offset, self._maxCaretOffset)
-        self.accessible.text.setCaretOffset(newCaretOffset)
+        self.accessible.queryText().setCaretOffset(newCaretOffset)
 
     def getAttributeMask(self):
         """Creates a string which can be used as the attrOr field of brltty's
         write structure for the purpose of indicating text attributes and
         selection."""
 
-        text = self.accessible.text
-        if not text:
+        try:
+            text = self.accessible.queryText()
+        except NotImplementedError:
             return ''
 
         # Start with an empty mask.
@@ -448,7 +446,7 @@ class Text(Region):
         indicateAttr = (attrIndicator != settings.TEXT_ATTR_BRAILLE_NONE)
         indicateSel = (selIndicator != settings.BRAILLE_SEL_NONE)
 
-        if indicateAttr:
+        if indicateAttr and settings.enabledBrailledTextAttributes:
             # Identify what attributes the user cares about.  Also get the
             # default attributes for the text object because those attributes
             # may not be included as attributes for the character.
@@ -467,7 +465,8 @@ class Text(Region):
             # character that has that attribute]]]
             #
             adjustment = 0
-            if orca_state.locusOfFocus.app.name == "soffice.bin":
+            appName = orca_state.locusOfFocus.getApplication().name
+            if appName.startswith("soffice"):
                 adjustment += 1
 
             i = self.lineOffset
@@ -575,7 +574,7 @@ class ReviewText(Region):
         been scrolled off the display."""
 
         newCaretOffset = self.lineOffset + offset
-        self.accessible.text.setCaretOffset(newCaretOffset)
+        self.accessible.queryText().setCaretOffset(newCaretOffset)
 
 class Line:
     """A horizontal line on the display.  Each Line is composed of a sequential
@@ -832,12 +831,13 @@ def refresh(panToCursor=True, targetCursorCell=0):
     else:
         cursorCell += 1 # Normalize to 1-based offset
 
-    debug.println(debug.LEVEL_INFO, "BRAILLE LINE:  '%s'" % string)
-    debug.println(debug.LEVEL_INFO, "     VISIBLE:  '%s', cursor=%d" \
-                  % (string[startPos:endPos], cursorCell))
-
-    log.info("refresh line='%s', visible='%s', cursor=%d" \
-             % (string, string[startPos:endPos], cursorCell))
+    logLine = "BRAILLE LINE:  '%s'" % string
+    debug.println(debug.LEVEL_INFO, logLine)
+    log.info(logLine)
+    logLine = "     VISIBLE:  '%s', cursor=%d" % \
+                    (string[startPos:endPos], cursorCell)
+    debug.println(debug.LEVEL_INFO, logLine)
+    log.info(logLine)
 
     string = string.decode("UTF-8")
     substring = string[startPos:endPos].encode("UTF-8")
@@ -1065,27 +1065,27 @@ def _processBrailleEvent(command):
     return consumed
 
 def _brlAPIKeyReader(source, condition):
-        """Method to read a key from the BrlAPI bindings.  This is a
-        gobject IO watch handler.
-        """
-        key = brlAPI.readKey(False)
-        if key:
-            flags = key >> 32
-            lower = key & 0xFFFFFFFF
-            keyType = lower >> 29
-            keyCode = lower & 0x1FFFFFFF
+    """Method to read a key from the BrlAPI bindings.  This is a
+    gobject IO watch handler.
+    """
+    key = brlAPI.readKey(False)
+    if key:
+        flags = key >> 32
+        lower = key & 0xFFFFFFFF
+        keyType = lower >> 29
+        keyCode = lower & 0x1FFFFFFF
 
-            # [[TODO: WDW - HACK If we have a cursor routing key, map
-            # it back to the code we used to get with earlier versions
-            # of BrlAPI (i.e., bit 0x100 was the indicator of a cursor
-            # routing key instead of 0x1000).  This may change before
-            # the offical BrlAPI Python bindings are released.]]]
-            #
-            if keyCode & 0x10000:
-                keyCode = 0x100 | (keyCode & 0xFF)
-            if keyCode:
-                _processBrailleEvent(keyCode)
-        return brlAPIRunning
+        # [[TODO: WDW - HACK If we have a cursor routing key, map
+        # it back to the code we used to get with earlier versions
+        # of BrlAPI (i.e., bit 0x100 was the indicator of a cursor
+        # routing key instead of 0x1000).  This may change before
+        # the offical BrlAPI Python bindings are released.]]]
+        #
+        if keyCode & 0x10000:
+            keyCode = 0x100 | (keyCode & 0xFF)
+        if keyCode:
+            _processBrailleEvent(keyCode)
+    return brlAPIRunning
 
 def setupKeyRanges(keys):
     """Hacky method to tell BrlTTY what to send and not send us via
