@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2005-2006 Sun Microsystems Inc.
+# Copyright 2005-2007 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -22,24 +22,15 @@ level, which is held in the debugLevel field.  All other methods take
 a debug level, which is compared to the current debug level to
 determine if the content should be output."""
 
-__id__        = "$Id: debug.py 2160 2007-03-12 17:18:51Z richb $"
-__version__   = "$Revision: 2160 $"
-__date__      = "$Date: 2007-03-12 13:18:51 -0400 (Mon, 12 Mar 2007) $"
-__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__id__        = "$Id: debug.py 3189 2007-10-26 15:10:12Z wwalker $"
+__version__   = "$Revision: 3189 $"
+__date__      = "$Date: 2007-10-26 11:10:12 -0400 (Fri, 26 Oct 2007) $"
+__copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
-import logging
 import sys
-logging.basicConfig()
-
-log = logging.getLogger()
-
-# Set the level to INFO if you want the logging of speech and braille
-# output.
-#
-#log.setLevel(logging.INFO)
-
 import traceback
+import pyatspi
 
 # Used to turn off all debugging.
 #
@@ -190,7 +181,7 @@ def printObjectEvent(level, event, sourceInfo=None):
     println(level, text)
 
     if sourceInfo:
-        println(level, "             " + sourceInfo)
+        println(level, "             %s" % sourceInfo)
 
 def printInputEvent(level, string):
     """Prints out an input event.  The given level may be overridden
@@ -215,11 +206,57 @@ def printDetails(level, indent, accessible, includeApp=True):
     """
 
     if level >= debugLevel and accessible:
-        println(level, accessible.toString(indent, includeApp))
+        println(level, getAccessibleDetails(accessible, indent, includeApp))
+
+def getAccessibleDetails(acc, indent="", includeApp=True):
+    """Returns a string, suitable for printing, that describes the
+    given accessible.
+
+    Arguments:
+    - indent: A string to prefix the output with
+    - includeApp: If True, include information about the app
+                  for this accessible.
+    """
+
+    if includeApp:
+        app = acc.getApplication()
+        if app:
+            string = indent + "app.name='%s' " % app.name
+        else:
+            string = indent + "app=None "
+    else:
+        string = indent
+
+    # create the States string
+    stateSet = acc.getState()
+    states = stateSet.getStates()
+    state_strings = []
+    for state in states:
+        state_strings.append(pyatspi.stateToString(state))
+    state_string = ' '.join(state_strings)
+
+    # create the relations string
+    relations = acc.getRelationSet()
+    if relations:
+        relation_strings = []
+        for relation in relations:
+            relation_strings.append( \
+                          pyatspi.relationToString(relation.getRelationType()))
+        rel_string = ' '.join(relation_strings)
+    else:
+        rel_string = ''
+
+    string += "name='%s' role='%s' state='%s' relations='%s'" \
+              % (acc.name or 'None', acc.getRoleName(),
+                 state_string, rel_string)
+
+    return string
+
 
 # The following code has been borrowed from the following URL:
 # 
-# http://www.dalkescientific.com/writings/diary/archive/2005/04/20/tracing_python_code.html
+# http://www.dalkescientific.com/writings/diary/archive/ \
+#                                     2005/04/20/tracing_python_code.html
 #
 import linecache
 
