@@ -19,9 +19,9 @@
 
 """Provides the FocusTrackingPresenter for Orca."""
 
-__id__        = "$Id: focus_tracking_presenter.py 2573 2007-07-31 16:11:39Z richb $"
-__version__   = "$Revision: 2573 $"
-__date__      = "$Date: 2007-07-31 12:11:39 -0400 (Tue, 31 Jul 2007) $"
+__id__  = "$Id: focus_tracking_presenter.py 3192 2007-10-26 19:41:18Z eitani $"
+__version__   = "$Revision: 3192 $"
+__date__      = "$Date: 2007-10-26 15:41:18 -0400 (Fri, 26 Oct 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -30,21 +30,18 @@ import Queue
 import threading
 import time
 
-import atspi
+import pyatspi
 import braille
 import default
 import debug
 import input_event
 import orca_state
 import presentation_manager
-import rolenames
 import settings
 import speech
 
 from orca_i18n import _ # for gettext support
 
-import CORBA
-
 class FocusTrackingPresenter(presentation_manager.PresentationManager):
     """Maintain a set of scripts for all running applications, and
     also keeps the notion of an activeScript.  All object events are
@@ -65,7 +62,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         # is what delves them out, we keep at most one listener to avoid
         # receiving the same event twice in a row.
         #
-        self.registry        = atspi.Registry()
+        self.registry        = pyatspi.Registry
         self._knownScripts   = {}
         self._knownAppSettings = {}
         self._oldAppSettings = None
@@ -207,11 +204,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         # default script. Note that this search is restricted to the "orca"
         # package for now.
         #
-        if (not script) \
-            and app \
-            and app.__dict__.has_key("toolkitName") \
-            and app.toolkitName:
-
+        if (not script) and app and getattr(app, "toolkitName", None):
             try:
                 debug.println(
                     debug.LEVEL_FINE,
@@ -299,19 +292,6 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         """
 
         objectsRemoved = 0
-        for obj in atspi.Accessible._cache.values():
-            try:
-                if obj.state.count(atspi.Accessibility.STATE_DEFUNCT):
-                    atspi.Accessible.deleteAccessible(obj)
-                    objectsRemoved += 1
-                else:
-                    # Try to force a COMM_FAILURE
-                    #
-                    obj.toString()
-            except CORBA.COMM_FAILURE:
-                atspi.Accessible.deleteAccessible(obj)
-                objectsRemoved += 1
-
         debug.println(debug.LEVEL_FINEST,
                       "_cleanupCache: %d objects removed." % objectsRemoved)
 
@@ -321,7 +301,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         gc.collect()
         for obj in gc.garbage:
             try:
-                if isinstance(obj, atspi.Accessible):
+                if isinstance(obj, pyatspi.Accessibility.Accessible):
                     gc.garbage.remove(obj)
                     obj.__del__()
             except:
@@ -340,19 +320,10 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         # didn't reclaim this time.
         #
         try:
-            apps = []
-            desktop = self.registry.desktop
-            for i in range(0, desktop.childCount):
-                try:
-                    acc = desktop.getChildAtIndex(i)
-                    app = atspi.Accessible.makeAccessible(acc)
-                    if app:
-                        apps.insert(0, app)
-                except:
-                    pass
+            desktop = self.registry.getDesktop(0)
 
             for app in self._knownScripts.keys():
-                if apps.count(app) == 0:
+                if app not in desktop:
                     script = self._knownScripts[app]
                     self._deregisterEventListeners(script)
 
@@ -362,7 +333,6 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                     # cycle of the application referring to itself.
                     #
                     del self._knownScripts[app]
-                    app.app = None
                     del app
                     del script
         except:
@@ -384,7 +354,6 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         app = script.app
         settingsPackages = settings.settingsPackages
         moduleName = settings.getScriptModuleName(app)
-        module = None
 
         if moduleName and len(moduleName):
             for package in settingsPackages:
@@ -482,14 +451,15 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         # this problem.]]]
         #
         try:
-            if event.source.role == rolenames.ROLE_TOOL_TIP:
+            if event.source.getRole() == pyatspi.ROLE_TOOL_TIP:
                 # Check that it's okay to present tool tips. Always present
                 # tooltips initiated by the user pressing Control-F1 on the
                 # keyboard.
                 #
                 if isinstance(orca_state.lastInputEvent, \
                               input_event.KeyboardEvent):
-                    if not orca_state.lastNonModifierKeyEvent.event_string == "F1":
+                    if not orca_state.lastNonModifierKeyEvent.event_string \
+                                                                     == "F1":
                         return
 
                     # Mouse move events don't update orca_state.lastInputEvent
@@ -512,9 +482,8 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         # generally do not like object:children-changed:remove events,
         # either.
         #
-        if event.type == "object:children-changed:remove":
-            if event.source == atspi.Accessible.makeAccessible(
-                                   self.registry.desktop):
+        if event.type.startswith("object:children-changed:remove"):
+            if event.source == self.registry.getDesktop(0):
                 self._reclaimScripts()
                 self._cleanupCache()
                 if settings.debugMemoryUsage:
@@ -525,12 +494,12 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
             # We don't want to touch a defunct object.  It's useless and it
             # can cause hangs.
             #
-            if event.source \
-               and event.source.state.count(atspi.Accessibility.STATE_DEFUNCT):
-                debug.println(debug.LEVEL_FINEST,
-                              "IGNORING DEFUNCT OBJECT")
-                atspi.Accessible.deleteAccessible(event.source)
-                return
+            if event.source:
+                state = event.source.getState()
+                if state.contains(pyatspi.STATE_DEFUNCT):
+                    debug.println(debug.LEVEL_FINEST,
+                                  "IGNORING DEFUNCT OBJECT")
+                    return
 
             if (not debug.eventDebugFilter) \
                 or (debug.eventDebugFilter \
@@ -538,17 +507,11 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                 if not event.type.startswith("mouse:"):
                     debug.printDetails(debug.LEVEL_FINEST, "    ", event.source)
 
-        except CORBA.COMM_FAILURE:
-            debug.printException(debug.LEVEL_WARNING)
-            debug.println(debug.LEVEL_FINEST,
-                          "COMM_FAILURE above while processing event: " \
-                          + event.type)
-        except CORBA.OBJECT_NOT_EXIST:
+        except LookupError:
             debug.printException(debug.LEVEL_WARNING)
             debug.println(debug.LEVEL_WARNING,
-                          "OBJECT_NOT_EXIST above while processing event: " \
-                          + event.type)
-            atspi.Accessible.deleteAccessible(event.source)
+                          "LookupError above while processing event: %s" %\
+                          event.type)
             return
         except:
             debug.printException(debug.LEVEL_WARNING)
@@ -572,12 +535,12 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         oldLocusOfFocus = orca_state.locusOfFocus
         try:
             # If we've received a mouse event, then don't try to get
-            # event.source.app because the top most parent is of role 
-            # unknown, which will cause an ERROR message to be displayed.
-            # See Orca bug #409731 for more details.
+            # event.source.getApplication() because the top most parent
+            # is of role unknown, which will cause an ERROR message to be
+            # displayed. See Orca bug #409731 for more details.
             #
             if not event.type.startswith("mouse:"):
-                s = self._getScript(event.source.app)
+                s = self._getScript(event.source.getApplication())
             else:
                 s = orca_state.activeScript
             if not s:
@@ -590,8 +553,8 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
 
         while s and retryCount <= s.commFailureAttemptLimit:
             try:
-                if not event.source.state.count( \
-                                      atspi.Accessibility.STATE_ICONIFIED):
+                state = event.source.getState()
+                if not state.contains(pyatspi.STATE_ICONIFIED):
 
                     # [[[TODO: WDW - HACK we look for frame that get
                     # focus: as a means to detect active scripts
@@ -602,38 +565,36 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                     # main window, which is a frame.]]]
                     #
                     if (event.type == "window:activate") \
-                       or ((event.type == "focus:") \
-                           and (event.source.role == rolenames.ROLE_FRAME)):
-
+                       or ((event.type.startswith("focus")) 
+                           and (event.source.getRole() == pyatspi.ROLE_FRAME)):
                         # We'll let someone else decide if it's important
                         # to stop speech or not.
                         #speech.stop()
                         debug.println(debug.LEVEL_FINE, "ACTIVE SCRIPT: " \
                                       + orca_state.activeScript.name)
 
-                        self.setActiveScript(self._getScript(event.source.app))
+                        self.setActiveScript(
+                            self._getScript(event.source.getApplication()))
 
                         # Load in the application specific settings for the
                         # app for this event (if found).
                         #
-                        appSettings = self.loadAppSettings( \
-                                                      orca_state.activeScript)
+                        self.loadAppSettings(orca_state.activeScript)
 
                         # Tell BrlTTY which commands we care about.
                         #
                         braille.setupKeyRanges(\
                             orca_state.activeScript.brailleBindings.keys())
-
                     s.processObjectEvent(event)
                     if retryCount:
                         debug.println(debug.LEVEL_WARNING,
                                       "  SUCCEEDED AFTER %d TRIES" % retryCount)
                 break
-            except CORBA.COMM_FAILURE:
+            except LookupError:
                 debug.printException(debug.LEVEL_WARNING)
                 debug.println(debug.LEVEL_WARNING,
-                              "COMM_FAILURE above while processing: " \
-                              + event.type)
+                              "LookupError above while processing: %s" % \
+                              event.type)
                 retryCount += 1
                 if retryCount <= s.commFailureAttemptLimit:
                     # We want the old locus of focus to be reset so
@@ -649,7 +610,6 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                     debug.println(debug.LEVEL_WARNING,
                                   "  GIVING UP AFTER %d TRIES" \
                                   % (retryCount - 1))
-                    atspi.Accessible.deleteAccessible(event.source)
             except:
                 debug.printException(debug.LEVEL_WARNING)
                 break
@@ -661,17 +621,6 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         - e: an at-spi event.
         """
 
-        # Uncomment these lines if you want to see what it's like without
-        # the queue.
-        #
-        #if isinstance(e, input_event.KeyboardEvent):
-        #    self._processKeyboardEvent(e)
-        #elif isinstance(e, input_event.BrailleEvent):
-        #    self._processBrailleEvent(e)
-        #else:
-        #    self._processObjectEvent(atspi.Event(e))
-        #return
-
         if settings.debugEventQueue:
             if self._enqueueEventCount:
                 debug.println(debug.LEVEL_ALL,
@@ -682,11 +631,11 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
 
         event = None
         if isinstance(e, input_event.KeyboardEvent):
-            if e.type == atspi.Accessibility.KEY_PRESSED_EVENT:
+            if e.type == pyatspi.KEY_PRESSED_EVENT:
                 debug.println(debug.LEVEL_ALL,
                               "----------> QUEUEING KEYPRESS '%s' (%d)"
                               % (e.event_string, e.hw_code))
-            elif e.type == atspi.Accessibility.KEY_RELEASED_EVENT:
+            elif e.type == pyatspi.KEY_RELEASED_EVENT:
                 debug.println(debug.LEVEL_ALL,
                               "----------> QUEUEING KEYRELEASE '%s' (%d)"
                               % (e.event_string, e.hw_code))
@@ -696,10 +645,9 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                           "----------> QUEUEING BRAILLE COMMAND %d" % e.event)
             event = e
         else:
-            # We ignore defunct objects and let the atspi module take
-            # care of them for us.
+            # We ignore defunct objects.
             #
-            if (e.type == "object:state-changed:defunct"):
+            if e.type.startswith("object:state-changed:defunct"):
                 if settings.debugEventQueue:
                     self._enqueueEventCount -= 1
                 return
@@ -707,10 +655,9 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
             # We also generally do not like
             # object:property-change:accessible-parent events because
             # they indicate something is now whacked with the
-            # hierarchy, so we just ignore them and let the atspi
-            # module take care of it for us.
+            # hierarchy.
             #
-            if e.type == "object:property-change:accessible-parent":
+            if e.type.startswith("object:property-change:accessible-parent"):
                 if settings.debugEventQueue:
                     self._enqueueEventCount -= 1
                 return
@@ -718,21 +665,18 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
             # At this point in time, we only care when objects are
             # removed from the desktop.
             #
-            if (e.type == "object:children-changed:remove") \
-                and (e.source != self.registry.desktop):
+            if e.type.startswith("object:children-changed:remove") \
+                and (e.source != self.registry.getDesktop(0)):
                 if settings.debugEventQueue:
                     self._enqueueEventCount -= 1
                 return
 
-            # We create the event here because it will ref everything
-            # we want it to ref, thus allowing things to survive until
-            # they are processed on the gidle thread.
-            #
             # If the event doesn't have a source or that source is not marked
             # valid, then we don't care about this event. Just return.
             #
-            event = atspi.Event(e)
-            if not event.source or not event.source.valid:
+            
+            event = e
+            if not event.source:
                 debug.println(debug.LEVEL_FINEST,
                       "---------> IGNORING INVALID EVENT %s" % e.type)
                 if settings.debugEventQueue:
@@ -761,7 +705,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
             if settings.debugEventQueue:
                 debug.println(debug.LEVEL_ALL,
                               "           ...put complete")
-            if not self._gidleId:
+            if settings.asyncMode and (not self._gidleId):
                 if settings.gilSleepTime:
                     time.sleep(settings.gilSleepTime)
                 self._gidleId = gobject.idle_add(self._dequeueEvent)
@@ -773,6 +717,10 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
             if settings.debugEventQueue:
                 debug.println(debug.LEVEL_ALL,
                               "           ...released")
+
+            if not settings.asyncMode:
+                self._dequeueEvent()
+
         if settings.debugEventQueue:
             self._enqueueEventCount -= 1
 
@@ -804,12 +752,12 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
             event = self._eventQueue.get_nowait()
 
             if isinstance(event, input_event.KeyboardEvent):
-                if event.type == atspi.Accessibility.KEY_PRESSED_EVENT:
+                if event.type == pyatspi.KEY_PRESSED_EVENT:
                     debug.println(debug.LEVEL_ALL,
                                   "DEQUEUED KEYPRESS '%s' (%d) <----------" \
                                   % (event.event_string, event.hw_code))
                     pressRelease = "PRESS"
-                elif event.type == atspi.Accessibility.KEY_RELEASED_EVENT:
+                elif event.type == pyatspi.KEY_RELEASED_EVENT:
                     debug.println(debug.LEVEL_ALL,
                                   "DEQUEUED KEYRELEASE '%s' (%d) <----------" \
                                   % (event.event_string, event.hw_code))
@@ -1004,12 +952,18 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
             # Generate a fake window activation event so the application
             # can tell the user about itself.
             #
-            e = atspi.Event()
-            e.source   = win._acc
-            e.type     = "window:activate"
-            e.detail1  = 0
-            e.detail2  = 0
-            e.any_data = None
+            class _FakeEvent:
+                pass
+            class _FakeData:
+                def value(self):
+                    return None
+            fe = _FakeEvent()
+            fe.source   = win
+            fe.type     = "window:activate"
+            fe.detail1  = 0
+            fe.detail2  = 0
+            fe.any_data = _FakeData()
+            e = pyatspi.event.Event(fe)
             self._enqueueEvent(e)
 
     def deactivate(self):
