@@ -19,18 +19,16 @@
 
 """Displays a GUI for the user to quit Orca."""
 
-__id__        = "$Id: orca_quit.py,v 1.4 2006/10/02 19:28:57 richb Exp $"
-__version__   = "$Revision: 1.4 $"
-__date__      = "$Date: 2006/10/02 19:28:57 $"
+__id__        = "$Id: orca_quit.py 3077 2007-10-17 16:32:55Z richb $"
+__version__   = "$Revision: 3077 $"
+__date__      = "$Date: 2007-10-17 12:32:55 -0400 (Wed, 17 Oct 2007) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import os
 import sys
 import debug
-import gettext
 import gtk
-import gtk.glade
 import locale
 
 import orca
@@ -39,8 +37,6 @@ import orca_state
 import platform
 import settings
 
-from orca_i18n import _  # for gettext support
-
 OS = None
 
 class orcaQuitGUI(orca_glade.GladeWrapper):
@@ -53,19 +49,20 @@ class orcaQuitGUI(orca_glade.GladeWrapper):
         already been created.
         """
 
+        quitDialog = self.get_widget("quitDialog")
+
         # Set the current time on the quit GUI dialog so that it'll
         # get focus. set_user_time is a new call in pygtk 2.9.2 or later.
         # It's surronded by a try/except block here so that if it's not found,
         # then we can fail gracefully.
         #
         try:
-            self.quitDialog.realize()
-            self.quitDialog.window.set_user_time(\
-                orca_state.lastInputEventTimestamp)
+            quitDialog.realize()
+            quitDialog.window.set_user_time(orca_state.lastInputEventTimestamp)
         except AttributeError:
             debug.printException(debug.LEVEL_FINEST)
 
-        self.quitDialog.show()
+        quitDialog.show()
 
     def quitNoButtonClicked(self, widget):
         """Signal handler for the "clicked" signal for the quitNoButton
@@ -77,7 +74,7 @@ class orcaQuitGUI(orca_glade.GladeWrapper):
         - widget: the component that generated the signal.
         """
 
-        self.quitDialog.hide()
+        self.get_widget("quitDialog").hide()
         if settings.showMainWindow:
             orca._showMainWindowGUI()
 
