@@ -19,16 +19,11 @@
 
 import orca.debug as debug
 import orca.default as default
-import orca.atspi as atspi
-import orca.input_event as input_event
-import orca.rolenames as rolenames
 import orca.braille as braille
 import orca.orca_state as orca_state
 import orca.speech as speech
-import orca.settings as settings
-import orca.speechgenerator as speechgenerator
-import orca.braillegenerator as braillegenerator
 import orca.eventsynthesizer as eventsynthesizer
+import pyatspi
 
 from orca.orca_i18n import _ # for gettext support
 
@@ -62,14 +57,14 @@ class Script(default.Script):
         # statusbar content which is the the information that sets the 
         # work online or work offline mode.
         #
-        if event.source.role != rolenames.ROLE_STATUSBAR:
+        if event.source.getRole() != pyatspi.ROLE_STATUS_BAR:
             default.Script.onNameChanged(self, event)
             return
 
-        rolesList = [rolenames.ROLE_PUSH_BUTTON,
-                     rolenames.ROLE_FILLER,
-                     rolenames.ROLE_FILLER,
-                     rolenames.ROLE_FRAME]
+        rolesList = [pyatspi.ROLE_PUSH_BUTTON,
+                     pyatspi.ROLE_FILLER,
+                     pyatspi.ROLE_FILLER,
+                     pyatspi.ROLE_FRAME]
 
         # We only speak the statusbar's changes when the application is 
         # with the focus and is the "work online/offline button is focused.
@@ -93,7 +88,7 @@ class Script(default.Script):
 
         debug.printObjectEvent(self.debugLevel,
                                event,
-                               event.source.toString())
+                               event.source)
 
         # Here we handle the case when focus is in the "Work online/offline" 
         # button near the status bar that has an image without a description.
@@ -102,10 +97,10 @@ class Script(default.Script):
         # associated with the button, which shows the online or offline 
         # work mode.
         #
-        rolesList = [rolenames.ROLE_PUSH_BUTTON,
-                     rolenames.ROLE_FILLER,
-                     rolenames.ROLE_FILLER,
-                     rolenames.ROLE_FRAME]
+        rolesList = [pyatspi.ROLE_PUSH_BUTTON,
+                     pyatspi.ROLE_FILLER,
+                     pyatspi.ROLE_FILLER,
+                     pyatspi.ROLE_FRAME]
 
         # We are checking if the button with the focus is the button to 
         # turn on/off the work mode in liferea. This push button is
@@ -113,37 +108,38 @@ class Script(default.Script):
         # (frame), inside a filler and inside another filler.
         #
         if self.isDesiredFocusedItem(event.source, rolesList):
-             # If we are focusing this button we construct a utterance and 
-             # a braille region to speak/braille "online/offline button".
-             # Here we declare utterances and add the localized string 
-             # "online/offline".
-             #
-             utterances = []
-             utterances.append(_("Work online / offline")) 
+            # If we are focusing this button we construct a utterance and 
+            # a braille region to speak/braille "online/offline button".
+            # Here we declare utterances and add the localized string 
+            # "online/offline".
+            #
+            utterances = []
+            utterances.append(_("Work online / offline")) 
 
-             # Here we extend the utterances with the speech generator for 
-             # the object with focus (the push button).
-             #
-             utterances.extend(speechGen.getSpeech(event.source,False))
+            # Here we extend the utterances with the speech generator for 
+            # the object with focus (the push button).
+            #
+            utterances.extend(speechGen.getSpeech(event.source, False))
 
-             # Finally we speak/braille the utterances/regions.
-             #
-             speech.speakUtterances(utterances)
+            # Finally we speak/braille the utterances/regions.
+            #
+            speech.speakUtterances(utterances)
            
-             regions = brailleGen.getBrailleRegions(event.source)
-             regions[0].insert(0, braille.Region(utterances[0] + " "))
-             braille.displayRegions(regions)
+            regions = brailleGen.getBrailleRegions(event.source)
+            regions[0].insert(0, braille.Region(utterances[0] + " "))
+            braille.displayRegions(regions)
            
-             return
+            return
 
         # Here we handle the case when the focus is in the headlines table.
         # See comment #3 of bug #350233.
         # http://bugzilla.gnome.org/show_bug.cgi?id=350233
         #
-        if orca_state.locusOfFocus.role == rolenames.ROLE_TABLE_COLUMN_HEADER:
-             table = event.source.parent
-             cells = self.findByRole(table, rolenames.ROLE_TABLE_CELL)
-             eventsynthesizer.clickObject(cells[1], 1)
-        
+        if orca_state.locusOfFocus.getRole() == \
+                                        pyatspi.ROLE_TABLE_COLUMN_HEADER:
+            table = event.source.parent
+            cells = self.findByRole(table, pyatspi.ROLE_TABLE_CELL)
+            eventsynthesizer.clickObject(cells[1], 1)
+
         default.Script.locusOfFocusChanged(self, event, 
                                            oldLocusOfFocus, newLocusOfFocus)
