@@ -19,17 +19,16 @@
 
 """Custom script for metacity."""
 
-__id__        = "$Id: metacity.py 2828 2007-09-13 19:26:33Z wwalker $"
-__version__   = "$Revision: 2828 $"
-__date__      = "$Date: 2007-09-13 15:26:33 -0400 (Thu, 13 Sep 2007) $"
+__id__        = "$Id: metacity.py 3093 2007-10-18 17:45:19Z shaeger $"
+__version__   = "$Revision: 3093 $"
+__date__      = "$Date: 2007-10-18 13:45:19 -0400 (Thu, 18 Oct 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import orca.braille as braille
 import orca.default as default
-import orca.rolenames as rolenames
-import orca.orca as orca
 import orca.speech as speech
+import pyatspi
 
 from orca.orca_i18n import _
 
@@ -67,7 +66,7 @@ class Script(default.Script):
         for app in self.getKnownApplications():
             i = 0
             while i < app.childCount:
-                win = app.child(i)
+                win = app.getChildAtIndex(i)
                 if win is None:
                     print "app error " + app.name
                 elif win.name == obj.name:
@@ -106,7 +105,7 @@ class Script(default.Script):
 
         # Ignore changes on the status bar.  We handle them in onTextInserted.
         #
-        if event.source.role != rolenames.ROLE_STATUSBAR:
+        if event.source.getRole() != pyatspi.ROLE_STATUS_BAR:
             default.Script.onNameChanged(self, event)
 
     def onStateChanged(self, event):
@@ -124,7 +123,7 @@ class Script(default.Script):
         # suddenly showing.  Then, we want to present it because we
         # typically do not get onTextInserted events at that time.
         #
-        if event.source.role != rolenames.ROLE_STATUSBAR:
+        if event.source.getRole() != pyatspi.ROLE_STATUS_BAR:
             default.Script.onStateChanged(self, event)
         elif (event.type.startswith("object:state-changed:showing")) \
             and event.detail1:
@@ -138,7 +137,7 @@ class Script(default.Script):
         - event: the Event
         """
 
-        if event.source.role != rolenames.ROLE_STATUSBAR:
+        if event.source.getRole() != pyatspi.ROLE_STATUS_BAR:
             default.Script.onTextInserted(self, event)
 
         self.presentStatusBar(event.source)
@@ -152,7 +151,7 @@ class Script(default.Script):
 
         # Ignore changes on the status bar.  We handle them in onTextInserted.
         #
-        if event.source.role != rolenames.ROLE_STATUSBAR:
+        if event.source.getRole() != pyatspi.ROLE_STATUS_BAR:
             default.Script.onTextDeleted(self, event)
 
     def onCaretMoved(self, event):
@@ -164,5 +163,5 @@ class Script(default.Script):
 
         # Ignore changes on the status bar.  We handle them in onTextInserted.
         #
-        if event.source.role != rolenames.ROLE_STATUSBAR:
+        if event.source.getRole() != pyatspi.ROLE_STATUS_BAR:
             default.Script.onCaretMoved(self, event)
