@@ -29,10 +29,7 @@ __author__    = "Tomas Cerha <cerha@brailcom.org>"
 __copyright__ = "Copyright (c) 2006, 2007 Brailcom, o.p.s."
 __license__   = "LGPL"
 
-import threading
 import gobject
-import time
-import re
 
 import debug
 import speechserver
@@ -40,7 +37,6 @@ import settings
 import orca
 from acss import ACSS
 from orca_i18n import _
-from speechserver import VoiceFamily
 
 try:
     import speechd
@@ -49,7 +45,9 @@ except:
 else:    
     _speechd_available = True
     try:
-        from speechd import CallbackType
+        # Done this way to prevent a pychecker warning.
+        key = "CallbackType"
+        setattr(speechd, key, getattr(speechd, key))
     except ImportError:
         _speechd_version_ok = False
     else:
@@ -108,10 +106,10 @@ class SpeechServer(speechserver.SpeechServer):
 
     def getSpeechServer(info=None):
         if info is not None:
-            id = info[1]
+            thisId = info[1]
         else:
-            id = SpeechServer.DEFAULT_SERVER_ID
-        return SpeechServer._getSpeechServer(id)
+            thisId = SpeechServer.DEFAULT_SERVER_ID
+        return SpeechServer._getSpeechServer(thisId)
     getSpeechServer = staticmethod(getSpeechServer)
 
     def shutdownActiveServers():
@@ -205,7 +203,7 @@ class SpeechServer(speechserver.SpeechServer):
         self._send_command(self._client.set_volume, volume)
 
     def _set_family(self, acss_family):
-        locale = acss_family[VoiceFamily.LOCALE]
+        locale = acss_family[speechserver.VoiceFamily.LOCALE]
         if locale:
             lang = locale.split('_')[0]
             if lang and len(lang) == 2:
@@ -216,17 +214,17 @@ class SpeechServer(speechserver.SpeechServer):
         except AttributeError:
             pass
         else:
-            name = acss_family[VoiceFamily.NAME]
+            name = acss_family[speechserver.VoiceFamily.NAME]
             if name != self._default_voice_name:
                 self._send_command(set_synthesis_voice, name)
             
     def _apply_acss(self, acss):
         current = self._current_voice_properties
-        for property, method in self._acss_manipulators:
-            value = acss.get(property)
-            if value is not None and current.get(property) != value:
+        for thisProperty, method in self._acss_manipulators:
+            value = acss.get(thisProperty)
+            if value is not None and current.get(thisProperty) != value:
                 method(value)
-                current[property] = value
+                current[thisProperty] = value
 
     def _speak(self, text, acss, **kwargs):
         if acss is not None:
@@ -306,9 +304,10 @@ class SpeechServer(speechserver.SpeechServer):
             pass
         else:
             voices += self._send_command(list_synthesis_voices)
-        families = [VoiceFamily({VoiceFamily.NAME: name,
-                                 #VoiceFamily.GENDER: VoiceFamily.MALE,
-                                 VoiceFamily.LOCALE: lang})
+        families = [speechserver.VoiceFamily({ \
+              speechserver.VoiceFamily.NAME: name,
+              #speechserver.VoiceFamily.GENDER: speechserver.VoiceFamily.MALE,
+              speechserver.VoiceFamily.LOCALE: lang})
                     for name, lang, dialect in voices]
         return families
 
