@@ -19,13 +19,13 @@
 
 """Speaks information about the current object of interest."""
 
-__id__        = "$Id: where_am_I.py 2955 2007-10-01 21:32:06Z joanied $"
-__version__   = "$Revision: 2955 $"
-__date__      = "$Date: 2007-10-01 17:32:06 -0400 (Mon, 01 Oct 2007) $"
+__id__        = "$Id: where_am_I.py 3178 2007-10-25 14:50:02Z joanied $"
+__version__   = "$Revision: 3178 $"
+__date__      = "$Date: 2007-10-25 10:50:02 -0400 (Thu, 25 Oct 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
-import atspi
+import pyatspi
 import debug
 import orca_state
 import rolenames
@@ -78,65 +78,63 @@ class WhereAmI:
            \n  orca-key=%s" % \
             (self._getObjLabel(obj),
              self._getObjName(obj),
-             obj.role,
+             obj.getRoleName(),
              self._script.getAcceleratorAndShortcut(obj),
              self._getObjLabel(obj.parent),
              self._getObjName(obj.parent),
-             obj.parent.role,
+             obj.parent.getRoleName(),
              doubleClick,
              orcaKey))
 
-        role = obj.role
+        role = obj.getRole()
 
         if orcaKey:
             self._processOrcaKey(obj, doubleClick)
 
-        elif role == rolenames.ROLE_CHECK_BOX:
+        elif role == pyatspi.ROLE_CHECK_BOX:
             self._speakCheckBox(obj, doubleClick)
 
-        elif role == rolenames.ROLE_RADIO_BUTTON:
+        elif role == pyatspi.ROLE_RADIO_BUTTON:
             self._speakRadioButton(obj, doubleClick)
 
-        elif role == rolenames.ROLE_COMBO_BOX:
+        elif role == pyatspi.ROLE_COMBO_BOX:
             self._speakComboBox(obj, doubleClick)
 
-        elif role == rolenames.ROLE_SPIN_BUTTON:
+        elif role == pyatspi.ROLE_SPIN_BUTTON:
             self._speakSpinButton(obj, doubleClick)
 
-        elif role == rolenames.ROLE_PUSH_BUTTON:
+        elif role == pyatspi.ROLE_PUSH_BUTTON:
             self._speakPushButton(obj, doubleClick)
 
-        elif role == rolenames.ROLE_SLIDER:
+        elif role == pyatspi.ROLE_SLIDER:
             self._speakSlider(obj, doubleClick)
 
-        elif role in [rolenames.ROLE_MENU,
-                      rolenames.ROLE_MENU_ITEM,
-                      rolenames.ROLE_CHECK_MENU,
-                      rolenames.ROLE_CHECK_MENU_ITEM,
-                      rolenames.ROLE_RADIO_MENU,
-                      rolenames.ROLE_RADIO_MENU_ITEM]:
+        elif role in [pyatspi.ROLE_MENU,
+                      pyatspi.ROLE_MENU_ITEM,
+                      pyatspi.ROLE_CHECK_MENU_ITEM,
+                      pyatspi.ROLE_RADIO_MENU_ITEM]:
             self._speakMenuItem(obj, doubleClick)
 
-        elif role == rolenames.ROLE_PAGE_TAB:
+        elif role == pyatspi.ROLE_PAGE_TAB:
             self._speakPageTab(obj, doubleClick)
 
-        elif role in [rolenames.ROLE_TEXT,
-                      rolenames.ROLE_TERMINAL]:
+        elif role in [pyatspi.ROLE_TEXT,
+                      pyatspi.ROLE_TERMINAL]:
             self._speakText(obj, doubleClick)
 
-        elif role == rolenames.ROLE_TABLE_CELL:
+        elif role == pyatspi.ROLE_TABLE_CELL:
             self._speakTableCell(obj, doubleClick)
 
-        elif role == rolenames.ROLE_LIST_ITEM:
+        elif role == pyatspi.ROLE_LIST_ITEM:
             self._speakListItem(obj, doubleClick)
 
-        elif role == rolenames.ROLE_PARAGRAPH:
+        elif role == pyatspi.ROLE_PARAGRAPH:
             self._speakParagraph(obj, doubleClick)
 
-        elif role == rolenames.ROLE_ICON:
+        elif role == pyatspi.ROLE_ICON:
             self._speakIconPanel(obj, doubleClick)
             
-        elif role == rolenames.ROLE_LINK:
+        elif role == pyatspi.ROLE_LINK:
             self._speakLink(obj, doubleClick)
 
         else:
@@ -164,7 +162,8 @@ class WhereAmI:
                rolenames.getSpeechForRoleName(obj)
         utterances.append(text)
 
-        if obj.state.count(atspi.Accessibility.STATE_CHECKED):
+        state = obj.getState()
+        if state.contains(pyatspi.STATE_CHECKED):
             # Translators: this represents the state of a checkbox.
             #
             text = _("checked")
@@ -201,7 +200,8 @@ class WhereAmI:
                rolenames.getSpeechForRoleName(obj)
         utterances.append(text)
 
-        if obj.state.count(atspi.Accessibility.STATE_CHECKED):
+        state = obj.getState()
+        if state.contains(pyatspi.STATE_CHECKED):
             # Translators: this is in reference to a radio button being
             # selected or not.
             #
@@ -250,7 +250,7 @@ class WhereAmI:
         utterances.append(name)
 
         # child(0) is the popup list
-        text = self._getPositionInList(obj.child(0), name)
+        text = self._getPositionInList(obj[0], name)
         utterances.append(text)
 
         text = self._getObjMnemonic(obj)
@@ -361,12 +361,14 @@ class WhereAmI:
         text = self._getObjLabelAndName(obj)
         utterances.append(text)
 
-        if obj.role != rolenames.ROLE_MENU_ITEM:
+        state = obj.getState()
+
+        if obj.getRole() != pyatspi.ROLE_MENU_ITEM:
             text = rolenames.getSpeechForRoleName(obj)
             utterances.append(text)
 
-        if obj.role == rolenames.ROLE_CHECK_MENU_ITEM:
-            if obj.state.count(atspi.Accessibility.STATE_CHECKED):
+        if obj.getRole() == pyatspi.ROLE_CHECK_MENU_ITEM:
+            if state.contains(pyatspi.STATE_CHECKED):
                 # Translators: this represents the state of a checkbox.
                 #
                 text = _("checked")
@@ -376,8 +378,8 @@ class WhereAmI:
                 text = _("not checked")
             utterances.append(text)
 
-        elif obj.role == rolenames.ROLE_RADIO_MENU_ITEM:
-            if obj.state.count(atspi.Accessibility.STATE_CHECKED):
+        elif obj.getRole() == pyatspi.ROLE_RADIO_MENU_ITEM:
+            if state.contains(pyatspi.STATE_CHECKED):
                 # Translators: this is in reference to a radio button being
                 # selected or not.
                 #
@@ -402,7 +404,7 @@ class WhereAmI:
         # Alt FO for the open menu item in the File menu). We only
         # want to speak the shortcut associated with the menu item.
         #
-        if text and obj.parent and obj.parent.role == rolenames.ROLE_MENU:
+        if text and obj.parent and obj.parent.getRole() == pyatspi.ROLE_MENU:
             text = text[-1]
         
         utterances.append(text)
@@ -482,7 +484,7 @@ class WhereAmI:
         speech.speakUtterances(utterances)
 
         if doubleClick:
-            verbalizePunctuationStyle = savedStyle
+            settings.verbalizePunctuationStyle = savedStyle
 
         utterances = []
         if selected:
@@ -517,7 +519,7 @@ class WhereAmI:
 
         # Speak the first two items (and possibly the position)
         utterances = []
-        if obj.parent.role == rolenames.ROLE_TABLE_CELL:
+        if obj.parent.getRole() == pyatspi.ROLE_TABLE_CELL:
             obj = obj.parent
         parent = obj.parent
 
@@ -532,11 +534,12 @@ class WhereAmI:
 
         utterances = []
         if doubleClick:
-            table = parent.table
-            row = table.getRowAtIndex(orca_state.locusOfFocus.index)
+            table = parent.queryTable()
+            row = table.getRowAtIndex(
+              orca_state.locusOfFocus.getIndexInParent())
             # Translators: this in reference to a row in a table.
             #
-            text = _("row %d of %d") % ((row+1), parent.table.nRows)
+            text = _("row %d of %d") % ((row+1), table.nRows)
             utterances.append(text)
             speech.speakUtterances(utterances)
 
@@ -550,19 +553,24 @@ class WhereAmI:
         utterances = []
 
         if not doubleClick:
-            table = parent.table
-            if not table:
-                debug.println(self._debugLevel, "??? parent=%s" % parent.role)
+            try:
+                table = parent.queryTable()
+            except NotImplementedError:
+                debug.println(self._debugLevel, 
+                              "??? parent=%s" % parent.getRoleName())
                 return
+            else:
+                row = \
+                    table.getRowAtIndex(
+                       orca_state.locusOfFocus.getIndexInParent())
+                # Translators: this in reference to a row in a table.
+                #
+                text = _("row %d of %d") % ((row+1), table.nRows)
+                utterances.append(text)
 
-            row = table.getRowAtIndex(orca_state.locusOfFocus.index)
-            # Translators: this in reference to a row in a table.
-            #
-            text = _("row %d of %d") % ((row+1), parent.table.nRows)
-            utterances.append(text)
-
-        if obj.state.count(atspi.Accessibility.STATE_EXPANDABLE):
-            if obj.state.count(atspi.Accessibility.STATE_EXPANDED):
+        state = obj.getState()
+        if state.contains(pyatspi.STATE_EXPANDABLE):
+            if state.contains(pyatspi.STATE_EXPANDED):
                 # Translators: this represents the state of a node in a tree.
                 # 'expanded' means the children are showing.
                 # 'collapsed' means the children are not showing.
@@ -610,19 +618,19 @@ class WhereAmI:
         utterances.append(text)
 
         parent = obj.parent
-        relations = obj.relations
-        for relation in relations:
-            if relation.getRelationType() \
-                   == atspi.Accessibility.RELATION_NODE_CHILD_OF:
-                parent = atspi.Accessible.makeAccessible(relation.getTarget(0))
+        relationset = obj.getRelationSet()
+        for relation in relationset:
+            if relation.getRelationType() == pyatspi.RELATION_NODE_CHILD_OF:
+                parent = relation.getTarget(0)
                 break
 
         name = self._getObjName(obj)
         text = self._getPositionInList(parent, name)
         utterances.append(text)
 
-        if obj.state.count(atspi.Accessibility.STATE_EXPANDABLE):
-            if obj.state.count(atspi.Accessibility.STATE_EXPANDED):
+        state = obj.getState()
+        if state.contains(pyatspi.STATE_EXPANDABLE):
+            if state.contains(pyatspi.STATE_EXPANDED):
                 # Translators: this represents the state of a node in a tree.
                 # 'expanded' means the children are showing.
                 # 'collapsed' means the children are not showing.
@@ -672,9 +680,9 @@ class WhereAmI:
 
         selectedItems = []
         totalSelectedItems = 0
-        for i in range(0, childCount):
-            child = panel.child(i)
-            if child.state.count(atspi.Accessibility.STATE_SELECTED):
+        for child in panel:
+            state = child.getState()
+            if state.contains(pyatspi.STATE_SELECTED):
                 totalSelectedItems += 1
                 selectedItems.append(child)
 
@@ -765,7 +773,7 @@ class WhereAmI:
         Get the http header for a given uri and try to extract the size (Content-length).
         '''
         try:
-            x=urllib2.urlopen(uri)
+            x = urllib2.urlopen(uri)
             try:
                 return x.info()['Content-length']
             except KeyError:
@@ -818,7 +826,7 @@ class WhereAmI:
 
         if name and name != "None":
             text = name
-        # debug.println(self._debugLevel, "%s name=<%s>" % (obj.role, text))
+        debug.println(self._debugLevel, "%s name=<%s>" % (obj.getRole(), text))
 
         return text
 
@@ -831,7 +839,8 @@ class WhereAmI:
 
         if label and label != "None":
             text = label
-        # debug.println(self._debugLevel, "%s label=<%s>" % (obj.role, text))
+
+        debug.println(self._debugLevel, "%s label=<%s>" % (obj.getRole(), text))
 
         return text
 
@@ -847,11 +856,16 @@ class WhereAmI:
         else:
             text = label
 
-        if obj.text:
-            [string, startOffset, endOffset] = obj.text.getTextAtOffset(0,
-                atspi.Accessibility.TEXT_BOUNDARY_LINE_START)
+        try:
+            textObj = obj.queryText()
+        except NotImplementedError:
+            pass
+        else:
+            [string, startOffset, endOffset] = textObj.getTextAtOffset(0,
+                pyatspi.TEXT_BOUNDARY_LINE_START)
 
-            debug.println(self._debugLevel, "%s text=<%s>" % (obj.role, string))
+            debug.println(self._debugLevel, "%s text=<%s>" % \
+                            (obj.getRoleName(), string))
 
         return text.strip()
 
@@ -862,12 +876,10 @@ class WhereAmI:
         text = ""
         labelledBy = None
 
-        relations = obj.relations
+        relations = obj.getRelationSet()
         for relation in relations:
-            if relation.getRelationType() ==  \
-                   atspi.Accessibility.RELATION_LABELLED_BY:
-                labelledBy = atspi.Accessible.makeAccessible( \
-                                                      relation.getTarget(0))
+            if relation.getRelationType() ==  pyatspi.RELATION_LABELLED_BY:
+                labelledBy = relation.getTarget(0)
                 break
 
         if labelledBy:
@@ -875,8 +887,8 @@ class WhereAmI:
         else:
             parent = obj.parent
             while parent and (parent.parent != parent):
-                if parent.role in [rolenames.ROLE_PANEL,
-                                   rolenames.ROLE_FILLER]:
+                if parent.getRole() in [pyatspi.ROLE_PANEL,
+                                        pyatspi.ROLE_FILLER]:
                     label = self._getObjLabelAndName(parent)
                     if label and label != "":
                         text = label
@@ -893,13 +905,12 @@ class WhereAmI:
         position = -1
         total = -1
 
-        relations = obj.relations
+        relations = obj.getRelationSet()
         for relation in relations:
-            if relation.getRelationType() == atspi.Accessibility.RELATION_MEMBER_OF:
+            if relation.getRelationType() == pyatspi.RELATION_MEMBER_OF:
                 total = relation.getNTargets()
                 for i in range(0, total):
-                    target = atspi.Accessible.makeAccessible( \
-                                                     relation.getTarget(i))
+                    target = relation.getTarget(i)
                     if target == obj:
                         position = total - i
                         break
@@ -934,11 +945,11 @@ class WhereAmI:
                 break
 
         if not total:
-            for i in range(0, obj.childCount):
-                next = self._getObjName(obj.child(i))
+            for child in obj:
+                next = self._getObjName(child)
+                state = child.getState()
                 if next in ["", "Empty", "separator"] \
-                   or not obj.child(i).state.count( \
-                    atspi.Accessibility.STATE_VISIBLE):
+                   or not state.contains(pyatspi.STATE_VISIBLE):
                     continue
 
                 index += 1
@@ -947,6 +958,7 @@ class WhereAmI:
                 if next == name:
                     position = index
 
+
         if position >= 0:
             # Translators: this is an item in a list.
             #
@@ -959,13 +971,13 @@ class WhereAmI:
         if either exists.
         """
 
-        list = self._script.getAcceleratorAndShortcut(obj)
+        results = self._script.getAcceleratorAndShortcut(obj)
 
         text = ""
-        if not list[1]:
-            text = list[0]
+        if not results[1]:
+            text = results[0]
         else:
-            text = list[0] + " " +  list[1]
+            text = results[0] + " " +  results[1]
 
         return text
 
@@ -973,11 +985,11 @@ class WhereAmI:
         """Returns the accelerator for the object, if it exists.
         """
 
-        list = self._script.getAcceleratorAndShortcut(obj)
+        results = self._script.getAcceleratorAndShortcut(obj)
 
         text = ""
-        if list[0]:
-            text = list[0]
+        if results[0]:
+            text = results[0]
 
         return text
 
@@ -985,11 +997,11 @@ class WhereAmI:
         """Returns the shortcut for the object, if it exists.
         """
 
-        list = self._script.getAcceleratorAndShortcut(obj)
+        results = self._script.getAcceleratorAndShortcut(obj)
 
         text = ""
-        if list[1]:
-            text = list[1]
+        if results[1]:
+            text = results[1]
 
         return text
 
@@ -997,7 +1009,7 @@ class WhereAmI:
         """Returns the slider's current value and percentage.
         """
 
-        value = obj.value
+        value = obj.queryValue()
 
         currentValue = "%.1f" % value.currentValue
         percent = value.currentValue / value.maximumValue * 100
@@ -1024,21 +1036,21 @@ class WhereAmI:
         utterances = []
 
         parent = obj.parent
-        table = parent.table
-        if not table:
-            debug.println(self._debugLevel, "??? parent=%s" % parent.role)
+        try:
+            table = parent.queryTable()
+        except NotImplementedError:
+            debug.println(self._debugLevel, "??? parent=%s" % parent.getRole())
             return []
+        else:
+            row = table.getRowAtIndex(obj.getIndexInParent())
 
-        row = parent.table.getRowAtIndex(obj.index)
+            for i in range(0, table.nColumns):
+                acc = table.getAccessibleAt(row, i)
+                utterances.append(self._getTableCell(acc))
 
-        for i in range(0, parent.table.nColumns):
-            cell = parent.table.getAccessibleAt(row, i)
-            acc = atspi.Accessible.makeAccessible(cell)
-            utterances.append(self._getTableCell(acc))
+            debug.println(self._debugLevel, "row=<%s>" % utterances)
 
-        debug.println(self._debugLevel, "row=<%s>" % utterances)
-
-        return utterances
+            return utterances
 
     def _getTableCell(self, obj):
         """Get the speech utterances for a single table cell.
@@ -1046,14 +1058,16 @@ class WhereAmI:
 
         # Don't speak check box cells that area not checked.
         notChecked = False
-        action = obj.action
-        if action:
+        try:
+            action = obj.queryAction()
+        except NotImplementedError:
+            pass
+        else:
             for i in range(0, action.nActions):
                 if action.getName(i) == "toggle":
-                    obj.role = rolenames.ROLE_CHECK_BOX
-                    if not obj.state.count(atspi.Accessibility.STATE_CHECKED):
+                    state = obj.getState()
+                    if not state.contains(pyatspi.STATE_CHECKED):
                         notChecked = True
-                    obj.role = rolenames.ROLE_TABLE_CELL
                     break
 
         if notChecked:
@@ -1077,7 +1091,7 @@ class WhereAmI:
         """
 
         textContents = ""
-        textObj = obj.text
+        textObj = obj.queryText()
         nSelections = textObj.getNSelections()
         for i in range(0, nSelections):
             [startOffset, endOffset] = textObj.getSelection(i)
@@ -1113,7 +1127,8 @@ class WhereAmI:
         textContents = ""
         startOffset = 0
         endOffset = 0
-        if obj.text.getNSelections() > 0:
+        text = obj.queryText()
+        if text.getNSelections() > 0:
             [textContents, startOffset, endOffset] = \
                                             self._getTextSelection(obj)
 
@@ -1122,19 +1137,19 @@ class WhereAmI:
             morePossibleSelections = True
             while morePossibleSelections:
                 morePossibleSelections = False
-                for relation in current.relations:
+                for relation in current.getRelationSet():
                     if relation.getRelationType() == \
-                           atspi.Accessibility.RELATION_FLOWS_FROM:
-                        prevObj = atspi.Accessible.makeAccessible( \
-                                                  relation.getTarget(0))
-                        if prevObj.text.getNSelections() > 0:
+                           pyatspi.RELATION_FLOWS_FROM:
+                        prevObj = relation.getTarget(0)
+                        prevObjText = prevObj.queryText()
+                        if prevObjText.getNSelections() > 0:
                             [newTextContents, start, end] = \
                                          self._getTextSelection(prevObj)
-                            textContents = newTextContents + textContents
+                            textContents = newTextContents + " " + textContents
                             current = prevObj
                             morePossibleSelections = True
                         else:
-                            displayedText = prevObj.text.getText(0, -1)
+                            displayedText = prevObjText.getText(0, -1)
                             if len(displayedText) == 0:
                                 current = prevObj
                                 morePossibleSelections = True
@@ -1144,19 +1159,19 @@ class WhereAmI:
             morePossibleSelections = True
             while morePossibleSelections:
                 morePossibleSelections = False
-                for relation in current.relations:
+                for relation in current.getRelationSet():
                     if relation.getRelationType() == \
-                           atspi.Accessibility.RELATION_FLOWS_TO:
-                        nextObj = atspi.Accessible.makeAccessible( \
-                                                  relation.getTarget(0))
-                        if nextObj.text.getNSelections() > 0:
+                           pyatspi.RELATION_FLOWS_TO:
+                        nextObj = relation.getTarget(0)
+                        nextObjText = nextObj.queryText()
+                        if nextObjText.getNSelections() > 0:
                             [newTextContents, start, end] = \
                                          self._getTextSelection(nextObj)
-                            textContents += newTextContents
+                            textContents += " " + newTextContents
                             current = nextObj
                             morePossibleSelections = True
                         else:
-                            displayedText = nextObj.text.getText(0, -1)
+                            displayedText = nextObjText.getText(0, -1)
                             if len(displayedText) == 0:
                                 current = nextObj
                                 morePossibleSelections = True
@@ -1180,26 +1195,28 @@ class WhereAmI:
 
         currentSelected = False
         otherSelected = False
-        nSelections = obj.text.getNSelections()
+        text = obj.queryText()
+        nSelections = text.getNSelections()
         if nSelections:
             currentSelected = True
         else:
             otherSelected = False
-            displayedText = obj.text.getText(0, -1)
+            text = obj.queryText()
+            displayedText = text.getText(0, -1)
             if len(displayedText) == 0:
                 current = obj
                 morePossibleSelections = True
                 while morePossibleSelections:
                     morePossibleSelections = False
-                    for relation in current.relations:
+                    for relation in current.getRelationSet():
                         if relation.getRelationType() == \
-                               atspi.Accessibility.RELATION_FLOWS_FROM:
-                            prevObj = atspi.Accessible.makeAccessible( \
-                                                      relation.getTarget(0))
-                            if prevObj.text.getNSelections() > 0:
+                               pyatspi.RELATION_FLOWS_FROM:
+                            prevObj = relation.getTarget(0)
+                            prevObjText = prevObj.queryText()
+                            if prevObjText.getNSelections() > 0:
                                 otherSelected = True
                             else:
-                                displayedText = prevObj.text.getText(0, -1)
+                                displayedText = prevObjText.getText(0, -1)
                                 if len(displayedText) == 0:
                                     current = prevObj
                                     morePossibleSelections = True
@@ -1209,15 +1226,15 @@ class WhereAmI:
                 morePossibleSelections = True
                 while morePossibleSelections:
                     morePossibleSelections = False
-                    for relation in current.relations:
+                    for relation in current.getRelationSet():
                         if relation.getRelationType() == \
-                               atspi.Accessibility.RELATION_FLOWS_TO:
-                            nextObj = atspi.Accessible.makeAccessible( \
-                                                      relation.getTarget(0))
-                            if nextObj.text.getNSelections() > 0:
+                               pyatspi.RELATION_FLOWS_TO:
+                            nextObj = relation.getTarget(0)
+                            nextObjText = nextObj.queryText()
+                            if nextObjText.getNSelections() > 0:
                                 otherSelected = True
                             else:
-                                displayedText = nextObj.text.getText(0, -1)
+                                displayedText = nextObjText.getText(0, -1)
                                 if len(displayedText) == 0:
                                     current = nextObj
                                     morePossibleSelections = True
@@ -1234,12 +1251,10 @@ class WhereAmI:
         C. if the current line is blank/empty, 'blank'
         """
 
-        textObj = obj.text
+        textObj = obj.queryText()
         caretOffset = textObj.caretOffset
         textContents = ""
         selected = False
-        startSelOffset = -1
-        endSelOffset = -1
 
         nSelections = textObj.getNSelections()
         debug.println(self._debugLevel,
@@ -1257,7 +1272,7 @@ class WhereAmI:
             #
             [line, startOffset, endOffset] = textObj.getTextAtOffset(
                 textObj.caretOffset,
-                atspi.Accessibility.TEXT_BOUNDARY_LINE_START)
+                pyatspi.TEXT_BOUNDARY_LINE_START)
             debug.println(self._debugLevel, \
                 "_getTextContents: len=%d, start=%d, end=%d, line=<%s>" % \
                 (len(line), startOffset, endOffset, line))
@@ -1267,7 +1282,7 @@ class WhereAmI:
                 textContents = line
             else:
                 char = textObj.getTextAtOffset(caretOffset,
-                    atspi.Accessibility.TEXT_BOUNDARY_CHAR)
+                    pyatspi.TEXT_BOUNDARY_CHAR)
                 debug.println(self._debugLevel,
                     "_getTextContents: character=<%s>, start=%d, end=%d" % \
                     (char[0], char[1], char[2]))
@@ -1285,8 +1300,9 @@ class WhereAmI:
         """Adjust line to include attribute information.
         """
 
-        text = obj.text
-        if not text:
+        try:
+            text = obj.queryText()
+        except NotImplementedError:
             return ""
 
         newLine = ""
@@ -1405,24 +1421,23 @@ class WhereAmI:
 
         utterances = []
 
-        list = self._getFrameAndDialog(obj)
+        results = self._getFrameAndDialog(obj)
+
         if doubleClick:
-            if list[0]:
+            if results[0]:
                 self._statusBar = None
-                self._getStatusBar(list[0])
+                self._getStatusBar(results[0])
                 if self._statusBar:
                     self._speakStatusBar()
-            window = list[1] or list[0]
+            window = results[1] or results[0]
             if window:
-                self._defaultButton = None
-                self._getDefaultButton(window)
-                self._speakDefaultButton()
+                self._speakDefaultButton(window)
         else:
-            if list[0]:
-                text = self._getObjLabelAndName(list[0])
+            if results[0]:
+                text = self._getObjLabelAndName(results[0])
                 utterances.append(text)
-            if list[1]:
-                text = self._getObjLabelAndName(list[1])
+            if results[1]:
+                text = self._getObjLabelAndName(results[1])
                 utterances.append(text)
 
             debug.println(self._debugLevel, "titlebar utterances=%s" % \
@@ -1434,21 +1449,21 @@ class WhereAmI:
         the object.
         """
 
-        list = [None, None]
+        results = [None, None]
 
         parent = obj.parent
         while parent and (parent.parent != parent):
             #debug.println(self._debugLevel,
             #              "_getFrameAndDialog: parent=%s, %s" % \
-            #              (parent.role, self._getObjLabelAndName(parent)))
-            if parent.role == rolenames.ROLE_FRAME:
-                list[0] = parent
-            if parent.role in [rolenames.ROLE_DIALOG,
-                               rolenames.ROLE_FILE_CHOOSER]:
-                list[1] = parent
+            #              (parent.getRole(), self._getObjLabelAndName(parent)))
+            if parent.getRole() == pyatspi.ROLE_FRAME:
+                results[0] = parent
+            if parent.getRole() in [pyatspi.ROLE_DIALOG,
+                                    pyatspi.ROLE_FILE_CHOOSER]:
+                results[1] = parent
             parent = parent.parent
 
-        return list
+        return results
 
     def _getStatusBar(self, obj):
         """Gets the status bar.
@@ -1457,19 +1472,17 @@ class WhereAmI:
             return
 
         # debug.println(self._debugLevel, "_findStatusBar: ROOT=%s, %s" % \
-        #               (obj.role, self._getObjLabelAndName(obj)))
-
-        managesDescendants = obj.state.count(\
-            atspi.Accessibility.STATE_MANAGES_DESCENDANTS)
+        #               (obj.getRole(), self._getObjLabelAndName(obj)))
+        state = obj.getState()
+        managesDescendants = state.contains(pyatspi.STATE_MANAGES_DESCENDANTS)
         if managesDescendants:
             return
 
-        for i in range(0, obj.childCount):
-            child = obj.child(i)
+        for child in obj:
             # debug.println(self._debugLevel,
             #               "_findStatusBar: child=%s, %s" % \
-            #               (child.role, self._getObjLabelAndName(child)))
-            if child.role == rolenames.ROLE_STATUSBAR:
+            #               (child.getRole(), self._getObjLabelAndName(child)))
+            if child.getRole() == pyatspi.ROLE_STATUS_BAR:
                 self._statusBar = child
                 return
 
@@ -1489,8 +1502,7 @@ class WhereAmI:
             text = self._getObjName(self._statusBar)
             utterances.append(text)
         else:
-            for i in range(0, self._statusBar.childCount):
-                child = self._statusBar.child(i)
+            for child in self._statusBar:
                 text = self._getObjName(child)
                 utterances.append(text)
 
@@ -1504,40 +1516,48 @@ class WhereAmI:
         Arguments:
         - obj: the dialog box for which the default button should be obtained
         """
-        
-        if self._defaultButton:
-            return
 
-        for i in range(0, obj.childCount):
-            child = obj.child(i)
+        defaultButton = None
+        for child in obj:
             # debug.println(self._debugLevel,
             #               "_getDefaultButton: child=%s, %s" % \
-            #               (child.role, self._getObjLabelAndName(child)))
-            if child.role == rolenames.ROLE_PUSH_BUTTON \
-                and child.state.count(atspi.Accessibility.STATE_IS_DEFAULT):
-                self._defaultButton = child
-                return
+            #               (child.getRole(), self._getObjLabelAndName(child)))
+            state = child.getState()
+            if child.getRole() == pyatspi.ROLE_PUSH_BUTTON \
+                and state.contains(pyatspi.STATE_IS_DEFAULT):
+                defaultButton = child
+            else:
+                defaultButton = self._getDefaultButton(child)
 
-            if child.childCount > 0:
-                self._getDefaultButton(child)
+            if defaultButton:
+                break
 
-    def _speakDefaultButton(self):
+        return defaultButton
+
+    def _speakDefaultButton(self, obj):
         """Speaks the default button in a dialog.
+
+        Arguments:
+        - obj: the dialog box for which the default button should be obtained
         """
 
-        if not self._defaultButton \
-           or not self._defaultButton.state.count(\
-                                      atspi.Accessibility.STATE_SENSITIVE):
+        defaultButton = self._getDefaultButton(obj)
+
+        if not defaultButton:
             return
 
-        utterances =[]
+        state = defaultButton.getState()
+        if not state.contains(pyatspi.STATE_SENSITIVE):
+            return
+
+        utterances = []
 
         # Translators: The "default" button in a dialog box is the button
         # that gets activated when Enter is pressed anywhere within that
         # dialog box.
         #
         text = _("Default button is %s") % \
-               self._getObjName(self._defaultButton)
+               self._getObjName(defaultButton)
         utterances.append(text)
 
         debug.println(self._debugLevel, "default button utterances=%s" % \
